--[[
	LFMonitorDMNB
	Copyright  2011 Seahen (Arcanedamage@Stormscale (US))
	Text classification algorithm converted to Lua from DMNBtext for Weka
	Copyright  2008 Jiang Su

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

--[[
$Rev:: 33                    $:  Revision of last commit
$Author:: seahen             $:  Author of last commit
$Date:: 2011-09-19 03:43:58 #$:  Date of last commit
$WoW:: 4.0.6.13623           $:  Client of the last tests
]]

function LFMonitor:initClassifier(classifier)
	local ipairs            = ipairs
	local math_exp          = math.exp
	local math_log			= math.log

	classifier = classifier or {}
	classifier.worddb = classifier.worddb or {}
	classifier.classifierdb = classifier.classifierdb or { wordcount = 0, grossnham = 0, grossnspam = 0, nham = 0, nspam = 0, cspam = 0, cham = 0, cunsure = 0, probcache = {}}

	function classifier:getLogProb(wordstream)
		local db = self.classifierdb
		local worddb = self.worddb
		local nham = db.nham
		local nspam = db.nspam
		local wordcount = db.wordcount
		if wordcount <= 0 then
			return 0
		end
		local laplace
		if DMNB.STRICT_WEKA then
			laplace = math_log(wordcount)
		--  this correction seems too aggressive, probably because (unlike Weka users) we can't filter out rare words
		else
			laplace = math_log(#wordstream + 1)
		end
		local hamwords = (db.hamwords or 0) + (wordcount * laplace)
		local spamwords = (db.spamwords or 0) + (wordcount * laplace)
		local wordratio = math_log(spamwords / hamwords)
		local prob = math_log((nspam + 1) / (nham + 1))
		local word
		for _, word in ipairs(wordstream) do
			record = worddb[word]
			if record then
				prob = prob + math_log((record.spamcount + laplace) / (record.hamcount + laplace)) - wordratio
			end
		end
		return prob
	end

	function classifier:getProb(wordstream)
		local ratio = self:getLogProb(wordstream)
		if ratio > 709 then
			return 1
		else
			ratio = math_exp(ratio)
			return ratio/(1+ratio)
		end
	end

	function classifier:learn(wordstream, is_spam)
		local db = self.classifierdb
		local worddb = self.worddb
		local word
		for _, word in ipairs(wordstream) do
			if not worddb[word] then
				worddb[word] = { spamcount = 0, hamcount = 0 }
				db.wordcount = db.wordcount + 1
			end
		end
		local prob = self:getProb(wordstream)
		db.probcache = {}    -- nuke the prob cache
		if is_spam then
			prob = 2-prob*2
			db.nspam = db.nspam + prob
			db.grossnspam = db.grossnspam + 1
		else
			prob = prob*2
			db.nham  = db.nham + prob
			db.grossnham = db.grossnham + 1
		end
		for _, word in ipairs(wordstream) do
			local record = worddb[word]
			if is_spam then
				db.spamwords = (db.spamwords or 0) + prob
				record.spamcount = record.spamcount + prob
			else
				db.hamwords = (db.hamwords or 0) + prob
				record.hamcount  = record.hamcount + prob
			end
			worddb[word] = record
		end
	end

	return classifier
end
